/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Stefan Lienard
   co-programmer: ...
*/

#include "mapped_water.h"
#include "util/loading/load_param.h"
#include "util/loading/factory.h"
#include "material.h"


CREATE_FACTORY(MappedWater, CL_MAPPED_WATER);


MappedWater::MappedWater(const TiXmlElement* root)
 : texture(GL_TEXTURE_2D)
{
  this->setClassID(CL_MAPPED_WATER, "MappedWater");
  this->toList(OM_ENVIRON);

  if (root != NULL)
    this->loadParams(root);


    mat.setDiffuseMap(&this->texture, 0);
    // Change the view port to be the size of the texture we will render to
    // HACK
    this->textureSize = 512;
}

MappedWater::~MappedWater()
{

}

void MappedWater::loadParams(const TiXmlElement* root)
{
  WorldEntity::loadParams(root);

  LoadParam(root, "waterHeight", this, MappedWater, setHeight);
}


void MappedWater::draw() const
{
  glPushMatrix();

  /*
  glTranslatef (this->getAbsCoor ().x,
                this->getAbsCoor ().y,
                this->getAbsCoor ().z);
  */
  glTranslatef(0,this->waterHeight,0);

  //glEnable(GL_LIGHTING);


  //mat.setDiffuseMap("pictures/sky-replace.jpg", GL_TEXTURE_2D, 1);
  //mat.setTransparency(1.0);
  //mat.setDiffuse(1.0, 0, .1);


  // HACK

  //glDisable(GL_BLEND);
  //glActiveTexture(GL_TEXTURE0);
  //glBindTexture(GL_TEXTURE_2D, this->texture);

  mat.select();

  glBegin(GL_QUADS);
    glNormal3f(0,1,0);
    glTexCoord2f(0,0);
    //glMultiTexCoord2f(GL_TEXTURE1, 0,0);
    glVertex3f(0,0,0);
    glTexCoord2f(1,0);
    //glMultiTexCoord2f(GL_TEXTURE1, 1,0);
    glVertex3f(100,0,0);
    glTexCoord2f(1,1);
    //glMultiTexCoord2f(GL_TEXTURE1, 1,1);
    glVertex3f(100,0,-100);
    glTexCoord2f(0,1);
    //glMultiTexCoord2f(GL_TEXTURE1, 0,1);
    glVertex3f(0,0,-100);
  glEnd();

  glPopMatrix();
}

void MappedWater::tick(float dt)
{

}

void MappedWater::setHeight(float height)
{
  this->waterHeight = height;
}

void MappedWater::activateReflection()
{
  glPushAttrib(GL_VIEWPORT_BIT);
  glPushMatrix();

  glViewport(0,0, textureSize, textureSize);

  // Clear the color and depth bits, reset the matrix and position our camera.
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();
  //g_Camera.Look();


  // If our camera is above the water we will render the scene flipped upside down.
  // In order to line up the reflection nicely with the world we have to translate
  // the world to the position of our reflected surface, multiplied by two.
  //if(g_Camera.Position().y > waterHeight)
  //{
      // Translate the world, then flip it upside down
   //   glTranslatef(0.0f, waterHeight*2.0f, 0.0f);
    //  glScalef(1.0, -1.0, 1.0);

      // Since the world is updside down we need to change the culling to FRONT
      //glCullFace(GL_FRONT);

      // Set our plane equation and turn clipping on
     // double plane[4] = {0.0, 1.0, 0.0, -waterHeight};
   //   glEnable(GL_CLIP_PLANE0);
     // glClipPlane(GL_CLIP_PLANE0, plane);

      // Render the world upside down and clipped (only render the top flipped).
      // If we don't turn OFF caustics for the reflection texture we get horrible
      // artifacts in the water.  That is why we set bRenderCaustics to FALSE.
      //RenderWorld(false);

      // Turn clipping off
     // glDisable(GL_CLIP_PLANE0);

      // Restore back-face culling
    //  glCullFace(GL_BACK);
  //}
  /*else
  {
      // If the camera is below the water we don't want to flip the world,
      // but just render it clipped so only the top is drawn.
      double plane[4] = {0.0, 1.0, 0.0, waterHeight};
      glEnable(GL_CLIP_PLANE0);
      glClipPlane(GL_CLIP_PLANE0, plane);
      RenderWorld(true);
      glDisable(GL_CLIP_PLANE0);
  }*/

  // Restore the previous matrix
  //glPopMatrix();

    // Bind the current scene to our reflection texture
  //glBindTexture(GL_TEXTURE_2D, g_Texture[REFLECTION_ID]);




}

void MappedWater::deactivateReflection()
{
  glBindTexture(GL_TEXTURE_2D, texture.getTexture());

  mat.setDiffuseMap(&texture, 0);
  glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, textureSize, textureSize);

  glPopAttrib();
  glPopMatrix();
}

void MappedWater::activateRefraction()
{

}

void MappedWater::deactivateRefraction()
{

}
